"""
Computation dispersion curves and stress-displacement functions for PSV propagation.

:copyright:
    Andreas Fichtner (andreas.fichtner@erdw.ethz.ch), September 2022
:license:
    GNU General Public License, Version 3
    (http://www.gnu.org/copyleft/gpl.html)
"""

#- Packages.

import numpy as np
import matplotlib.pyplot as plt
import TOOLS.read_xml as rxml
import TOOLS.integrate_psv_alt as ipsv_alt
import TOOLS.integrate_psv as ipsv
import TOOLS.group_velocity_psv as vg
import TOOLS.kernels_psv as kpsv
import MODELS.models as m

#- Radius of the Earth (or other planetary body) [m].
Re = 6371000.0

def dispersion_psv(xml_input):
	"""
	Compute dispersion curves and displacement functions for PSV propagation.
	periods, phase_velocities = dispersion_psv(xml_input)
	Dispersion curves and displacement functions are written to files.
	"""

	#- read input ---------------------------------------------------------------------------------

	inp = rxml.read_xml(xml_input)
	inp = inp[1]

	verbose = int(inp["verbose"])

	model = inp["model"]

	write_output = int(inp["output"]["write_output"])

	output_directory = inp["output"]["directory"]

	tag = inp["output"]["tag"]

	r_min = float(inp["integration"]["starting_radius"])
	dr = float(inp["integration"]["radius_sampling"])

	f_min = float(inp["f_c_sampling"]["f_min"])
	f_max = float(inp["f_c_sampling"]["f_max"])
	df = float(inp["f_c_sampling"]["df"])

	c_min = float(inp["f_c_sampling"]["c_min"])
	c_max = float(inp["f_c_sampling"]["c_max"])
	dc = float(inp["f_c_sampling"]["dc"])

	#- initialisations ----------------------------------------------------------------------------

	#- Frequencies.
	#f = np.array([    8.0,     9.0,   10.0,   11.0,   12.0,   13.0,   14.0,   15.0,   16.0,  17.0,  18.0,  19.0,  20.0,   20.0,   21.0,  21.0,   22.0,   22.0,   23.0,  23.0,   24.0,  24.0,   25.0,  25.0,   25.0, 26.0,   26.0, 27.0,   27.0,   28.0,   28.0,   29.0,   29.0,   30.0,    30.0,   30.0,   31.0,    31.0,   31.0,   32.0,   32.0,    32.0,    33.0,   33.0,   33.0,   34.0,   34.0,   34.0,   35.0,    35.0,   35.0,   36.0,   36.0,   36.0,   37.0,  37.0,   37.0,   38.0,  38.0,   38.0,   39.0,  39.0,   39.0,     40.0,  40.0,   40.0,   41.0, 41.0,   42.0,   42.0,   43.0,   44.0,    44.0,   45.0,   45.0])

	#- Minimum and maximum trial wave speeds.
	#c_min = np.array([1546.6, 1503.9, 1434.5, 1393.4, 1315.2, 1224.6, 1140.9, 1050.7, 978.9, 928.0, 868.0, 823.9, 780.7, 1653.2, 749.0, 1584.3, 717.0, 1544.4, 683.6, 1458.2, 673.2, 1410.6, 662.8, 1365.9, 1761.2, 1314.2, 1734.2, 1276.6, 1708.6, 1230.8, 1683.5, 1209.1, 1652.0, 1167.86, 1612.3, 1896.5, 1149.95, 1579.7, 1912.4, 1106.4, 1552.34, 1901.98, 1068.8, 1514.7, 1892.5, 1041.1, 1473.8, 1872.1, 1014.52, 1443.9, 1881.2, 1028.1, 1413.7, 1851.4, 989.3, 1386.2, 1829.1, 948.2, 1352.9, 1809.0, 931.7, 1315.4, 1781.1, 897.5, 1314.5, 1764.3, 1319, 1760.8, 1241.8, 1753.4, 1715.4, 1702.31, 1903.9, 1671.9, 1905.2527])
	#c_max = np.array([1546.6, 1503.9, 1434.5, 1393.4, 1315.2, 1224.6, 1140.9, 1050.7, 978.9, 928.0, 868.0, 823.9, 780.7, 1653.2, 749.0, 1584.3, 717.0, 1544.4, 683.6, 1458.2, 673.2, 1410.6, 662.8, 1365.9, 1761.2, 1314.2, 1734.2, 1276.6, 1708.6, 1230.8, 1683.5, 1209.1, 1652.0, 1167.86, 1612.3, 1896.5, 1149.95, 1579.7, 1912.4, 1106.4, 1552.34, 1901.98, 1068.8, 1514.7, 1892.5, 1041.1, 1473.8, 1872.1, 1014.52, 1443.9, 1881.2, 1028.1, 1413.7, 1851.4, 989.3, 1386.2, 1829.1, 948.2, 1352.9, 1809.0, 931.7, 1315.4, 1781.1, 897.5, 1314.5, 1764.3, 1319, 1760.8, 1241.8, 1753.4, 1715.4, 1702.31, 1903.9, 1671.9, 1905.2527])


	#f = f[::2]
	#c_min = c_min[::2]
	#c_max = c_max[::2]

	#c_min -= 200.0
	#c_max += 200.0

	#c_min = np.zeros(len(c_min)) + 300.0
	#c_max = np.zeros(len(c_min)) + 1800.0

	f = np.arange(f_min,f_max + df,df,dtype=float)
	c = np.arange(c_min,c_max + dc,dc,dtype=float)

	omega = 2 * np.pi * f

	frequencies = []
	phase_velocities = []
	group_velocities = []

	if model == "EXTERNAL":
		r, rho, A, C, F, L, N = m.models(None, model)

	else:
		r = np.arange(r_min, Re + dr, dr, dtype=float)
		rho = np.zeros(len(r))
		A = np.zeros(len(r))
		C = np.zeros(len(r))
		F = np.zeros(len(r))
		L = np.zeros(len(r))
		N = np.zeros(len(r))

		for n in np.arange(len(r)):
			rho[n], A[n], C[n], F[n], L[n], N[n] = m.models(r[n], model)

	#- root-finding algorithm ---------------------------------------------------------------------

	max_modes=6


	#- loop over angular frequencies

	it = 0

	for _f in f:

		mode_count=0

		#print(it,_f,c_min[it],c_max[it])

		c = np.arange(c_min,c_max + dc,dc,dtype=float)
		
		_omega = 2.0 * np.pi * _f
		k = _omega / c

		#- loop over trial wavenumbers
		r_left = 0.0
		for n in np.arange(len(k)):

			#- compute vertical wave functions using alternative system
			r1, r2, r3, r4, r5, r = ipsv_alt.integrate_psv_alt(r_min, dr, _omega, k[n], r, rho, A, C, F, L, N)
			r_right = r2[len(r2)-1]

			#- check if there is a zero -----------------------------------------------------------
			if r_left * r_right < 0.0:

				mode_count+=1

				#- start bisection algorithm
				rr_left = r_left
				rr_right = r_right
				k_left = k[n-1]
				k_right = k[n]

				for i in np.arange(5):
					k_new = (k_left * np.abs(rr_right) + k_right * np.abs(rr_left)) / (np.abs(rr_left) + np.abs(rr_right))
					r1, r2, r3, r4, r5, r = ipsv_alt.integrate_psv_alt(r_min, dr, _omega, k_new, r, rho, A, C, F, L, N)
					rr = r2[len(r2)-1]
					if rr * rr_left < 0.0:
						k_right = k_new
						rr_right = rr
					elif rr * rr_right < 0.0:
						k_left = k_new
						rr_left = rr
					else:
						continue

				#==================================================================================
				#- compute final vertical wave functions and corresponding velocities and kernels -
				#==================================================================================

				#- compute final vertical wave functions using the original first-order system
				#- two independent solutions
				r11, r21, r31, r41, r = ipsv.integrate_psv(r_min, dr, _omega, k_new, r, rho, A, C, F, L, N, 1)
				r12, r22, r32, r42, r = ipsv.integrate_psv(r_min, dr, _omega, k_new, r, rho, A, C, F, L, N, 2)
				#- determine their weights via boundary condition (weight q1 is set to 1)
				nr = len(r) - 1
				q2 = -r21[nr] / r22[nr]
				#- final solution with boundary condition
				r1 = r11 + q2 * r12
				r2 = r21 + q2 * r22
				r3 = r31 + q2 * r32
				r4 = r41 + q2 * r42
				#- normalise to displacement amplitude at the surface
				mm = np.sqrt( r1[nr]**2 + r3[nr]**2 )
				if mm > 0.0:
					r1 = r1 / mm
					r2 = r2 / mm
					r3 = r3 / mm
					r4 = r4 / mm
				else:
					print('integration possibly unstable')

				#- phase velocity

				frequencies.append(_f)
				phase_velocities.append(_omega / k_new)

				#- group velocity

				U, I1, I3 = vg.group_velocity_psv(r1, r2, r3, r4, r, k_new, _omega/k_new, rho, A, C, F, L, N)
				group_velocities.append(U)

				#- kernels

				kpsv.kernels_psv(r, r1, r2, r3, r4, _omega, k_new, U, I1, rho, A, C, F, L, N, write_output, output_directory, tag)

				#==================================================================================
				#- screen output and displacement function files ----------------------------------
				#==================================================================================

				cc=_omega/k_new

				#- plot and print to screen
				if verbose==1:
					message=f"f={_f:.3f}"+f" Hz, c={cc:.3f}"+f" m/s, U={U:.3f}"+" m/s"
					print(message)

				#- write output
				if write_output==1:
					identifier = f"f={_f:.3f}"+f".c={cc:.3f}"
					fid = open(output_directory+"displacement_psv."+tag+"."+identifier,"w")
					fid.write("number of vertical sampling points\n")
					fid.write(str(len(r))+"\n")
					fid.write("radius, vertical displacement, horizontal displacement \n")
					for idx in np.arange(len(r)):
						fid.write(str(r[idx])+" "+str(r1[idx])+" "+str(r3[idx])+"\n")
					fid.close()

			r_left = r_right

			if mode_count>=max_modes: break

		#- Step to next frequency.

		it += 1

	#==============================================================================================
	#- output -------------------------------------------------------------------------------------
	#==============================================================================================

	#- dispersion curve ---------------------------------------------------------------------------

	if write_output==1:

		#- write dispersion curve
		fid = open(output_directory+"dispersion_psv."+tag,"w")
		for k in np.arange(len(frequencies)):
			fid.write(str(frequencies[k])+" "+str(phase_velocities[k])+" "+str(group_velocities[k])+"\n")
		fid.close()

	#- return -------------------------------------------------------------------------------------

	return frequencies, phase_velocities, group_velocities
